package com.bluetoothprinter_sp300.activity;
import com.bluetoothprinte_sp300.R;
import com.bluetoothprinter_sp300.bluetooth.BlueToothPrintFunction;
import com.bluetoothprinter_sp300.bluetooth.BluetoothService;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.widget.Toast;
public class BaseActivity extends Activity{
	private static final String TAG = "BluetoothPrint";
	private static final boolean D = true;
	// 请求的代码
	private static final int REQUEST_CONNECT_DEVICE = 1;
	private static final int REQUEST_ENABLE_BT = 2;
	// 连接的设备的名称
	public String mConnectedDeviceName = null;
	// 当地蓝牙适配器
	private BluetoothAdapter mBluetoothAdapter = null;
	public static BluetoothService mService = null;
	public Handler mHandler; 
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		// TODO Auto-generated method stub
		super.onCreate(savedInstanceState);
		
		
		// 得到本地的适配器
		mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

		if (mBluetoothAdapter == null) {
			Toast.makeText(this, "蓝牙不能使用！", Toast.LENGTH_LONG).show();
			// finish();
			return;
		}
	}

	@Override
	public void onStart() {
		super.onStart();
		if (D)
			Log.e(TAG, "++ ON START ++");
		// 判断蓝牙是否开启，如果蓝牙没有开启，请求它开启
		if (!mBluetoothAdapter.isEnabled()) {
			Intent enableIntent = new Intent(
					BluetoothAdapter.ACTION_REQUEST_ENABLE);
			startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
		} else {
			if (mService == null) {
				// 初始化BluetoothService执行蓝牙连接
				mService = new BluetoothService(this, mHandler);
			}
		}
		
	}

	@Override
	public synchronized void onResume() {
		super.onResume();
		if (D)
			Log.e(TAG, "+ ON RESUME +");
		if (mService != null) {
			if (mService.getState() == BluetoothService.STATE_NONE) {
				mService.start();
			}

		}
		
	}

	@Override
	public synchronized void onPause() {
		super.onPause();
		if (D)
			Log.e(TAG, "- ON PAUSE -");
	}

	@Override
	public void onStop() {
		super.onStop();
		if (D)
			Log.e(TAG, "-- ON STOP --");
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		// Stop the Bluetooth services
		
		if (D)
			Log.e(TAG, "--- ON DESTROY ---");
	}

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent data) {
		
		if (D)
			Log.d(TAG, "onActivityResult" + resultCode);
		switch (requestCode) {
		case REQUEST_CONNECT_DEVICE:
			// 当DeviceListActivity返回一个设备连接
			if (resultCode == Activity.RESULT_OK) {
				// 获取设备的MAC地址
				String address = data.getExtras().getString(
						DeviceListActivity.EXTRA_DEVICE_ADDRESS);
				// 得到BLuetoothDevice对象
				if (address == null) {
					break;
				}
				// 获取设备进行连接
				BluetoothDevice device = mBluetoothAdapter
						.getRemoteDevice(address);
				// 尝试连接到设备
				mService.connect(device);
			}
			break;
		case REQUEST_ENABLE_BT:
			// 当请求激活蓝牙的回报
			if (resultCode == Activity.RESULT_OK) {
				// 蓝牙现在是启用的,因此,建立一个聊天会话
				if (mService == null) {
					// 初始化BluetoothService执行蓝牙连接
					mService = new BluetoothService(this, mHandler);
				}
			} else {
				// 用户没有启用蓝牙或出错
				Log.d(TAG, "BT not enabled");
				Toast.makeText(this, R.string.bt_not_enabled_leaving,
						Toast.LENGTH_SHORT).show();
				finish();
			}
			break;
		
		}

	}
	public void connectBluetoothDevice()
	{
		// 启动DeviceListActivity，并等待返回
					Intent serverIntent = new Intent(this, DeviceListActivity.class);
					startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE);
	}
	public void disconnectBluetoothDevice()
	{
		if (mService != null)
			mService.stop();
	}
	
}
