package com.bluetoothprinter_sp300.activity;

import java.util.Set;

import com.bluetoothprinte_sp300.R;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;
public class DeviceListActivity extends Activity {
    
    private static final String TAG = "DeviceListActivity";
    private static final boolean D = true;

    // 返回额外的意图
    public static String EXTRA_DEVICE_ADDRESS = "device_address";

    // 成员字段
    private BluetoothAdapter mBtAdapter;
    private ArrayAdapter<String> mPairedDevicesArrayAdapter;
    private ArrayAdapter<String> mNewDevicesArrayAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // 设置这个窗口
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
        setContentView(R.layout.device_list);

        // 
       // setResult(Activity.RESULT_CANCELED);

        //初始化按钮来执行发现设备
        Button scanButton = (Button)this.findViewById(R.id.button_scan);
        scanButton.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                doDiscovery();
                v.setVisibility(View.GONE);
            }
        });
        //用于已经配对设备和对新发现的设备初始化阵列适配器
        mPairedDevicesArrayAdapter = new ArrayAdapter<String>(this, R.layout.device_name);
        mNewDevicesArrayAdapter = new ArrayAdapter<String>(this, R.layout.device_name);

        //找到并设置列表视图配对设备

        ListView pairedListView = (ListView) findViewById(R.id.paired_devices);
        pairedListView.setAdapter(mPairedDevicesArrayAdapter);
        pairedListView.setOnItemClickListener(mDeviceClickListener);

        // 找到并设置为新发现的设备的列表
        ListView newDevicesListView = (ListView) findViewById(R.id.new_devices);
        newDevicesListView.setAdapter(mNewDevicesArrayAdapter);
        newDevicesListView.setOnItemClickListener(mDeviceClickListener);
    
        // 当设备被发现注册广播
        IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
        this.registerReceiver(mReceiver, filter);
   
        //当发现已经完成了注册广播
        filter = new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
        this.registerReceiver(mReceiver, filter);

        // 让本地的蓝牙适配器
        mBtAdapter = BluetoothAdapter.getDefaultAdapter();

        // 给一组目前配对设备
        Set<BluetoothDevice> pairedDevices = mBtAdapter.getBondedDevices();

        // 如果有配对设备,添加每一个ArrayAdapter
        if (pairedDevices.size() > 0) {
            findViewById(R.id.title_paired_devices).setVisibility(View.VISIBLE);
            for (BluetoothDevice device : pairedDevices) {
                mPairedDevicesArrayAdapter.add(device.getName() + "\n" + device.getAddress());
            }
        } else {
            String noDevices = getResources().getText(R.string.none_paired).toString();
            mPairedDevicesArrayAdapter.add(noDevices);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();

        // 确保没有发现
        if (mBtAdapter != null) {
            mBtAdapter.cancelDiscovery();
        }

        // 注销广播听众
        this.unregisterReceiver(mReceiver);
    }

    /**
     * BluetoothAdapter开始发现设备
     */
    private void doDiscovery() {
        if (D) Log.d(TAG, "doDiscovery()");

        // 在标题上显示扫描的
        setProgressBarIndeterminateVisibility(true);
        setTitle(R.string.scanning);

        //打开新设备的字幕
        findViewById(R.id.title_new_devices).setVisibility(View.VISIBLE);

        // 如果已经发现就停止它
        if (mBtAdapter.isDiscovering()) {
            mBtAdapter.cancelDiscovery();
        }

        // 请求从BluetoothAdapter发现
        mBtAdapter.startDiscovery();
    }

    // 点击的监听器ListViews的所有设备中
    private OnItemClickListener mDeviceClickListener = new OnItemClickListener() {
        public void onItemClick(AdapterView<?> av, View v, int arg2, long arg3) {
            // 取消发现
            mBtAdapter.cancelDiscovery();

            //获取设备的MAC地址
            String info = ((TextView) v).getText().toString();
            String address = info.substring(info.length() - 17);

            //创建结果intent其中包括MAC地址
            Intent intent = new Intent();
            intent.putExtra(EXTRA_DEVICE_ADDRESS, address);

            //设置 结果完成 Activity
            setResult(Activity.RESULT_OK, intent);
            finish();
        }
    };

    // 当发现是完成了，BroadcastReceiver监听发现设备和更改标题
    private final BroadcastReceiver mReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();

            // 当发现一个设备
            if (BluetoothDevice.ACTION_FOUND.equals(action)) {
                //得到BluetoothDevice对象的意图
                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                // 如果它已经配对过,跳过它因为它是已经配对的
                if (device.getBondState() != BluetoothDevice.BOND_BONDED) {
                    mNewDevicesArrayAdapter.add(device.getName() + "\n" + device.getAddress());
                }
            //当发现完成后,改变Activity的名称
            } else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action)) {
                setProgressBarIndeterminateVisibility(false);
                setTitle(R.string.select_device);
                if (mNewDevicesArrayAdapter.getCount() == 0) {
                    String noDevices = getResources().getText(R.string.none_found).toString();
                    mNewDevicesArrayAdapter.add(noDevices);
                }
            }
        }
    };

}
