package com.bluetoothprinter_sp300.bluetooth;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.UUID;

import com.bluetoothprinter_sp300.Constant.MessageConstant;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothServerSocket;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

/**
 * 这个类处理所有工作的设置和管理蓝牙连接其他设备 它有一个线程,
 * 监听传入连接 一个线程用于连接设备和一个线程用于执行数据传输连接时
 * */
public class BluetoothService {
    
	private static final String TAG = "BluetoothService";
	private static final boolean D = true;
	// 在创建服务器套接字记录SDP的名字
	private static final String NAME = "BTPrinter";
	// 给这个应用程序设置一个独特的UUID
	private static final UUID MY_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");

	private final BluetoothAdapter mAdapter;
	private final Handler mHandler;
	private AcceptThread mAcceptThread;
	private ConnectThread mConnectThread;
	private ConnectedThread mConnectedThread;
	private int mState;
	public  byte[] BluetoothRecvBuf = new byte[1024];
	public  int BluetoothRecvBufLen ;
	// 当前连接状态的常量
	public static final int STATE_NONE = 0; // 默认的,什么都没做
	 public static final int STATE_LISTEN = 1; // 进入连接的监听
	public static final int STATE_CONNECTING = 2;// 启动一个外向连接
	public static final int STATE_CONNECTED = 3; // 连接到一个远程设备
    public String returnStr="";
	/**
	 * 构造函数。准备一个新的BTPrinter会话 ontext UI活动上下文 handler 处理程序发送消息发送回UI活动
	 */
	public BluetoothService(Context context, Handler handler) {
		mAdapter = BluetoothAdapter.getDefaultAdapter();
		mState = STATE_NONE;
		mHandler = handler;
	}

	/**
	 * 设置当前状态的连接 state 定义当前的连接状态  
	 */
	private synchronized void setState(int state) {
		if (D)
			Log.d(TAG, "setState()" + mState + "->" + state);
		mState = state;

		// 给新状态Handler让UI可以更新
		mHandler.obtainMessage(MessageConstant.MESSAGE_STATE_CHANGE, state, -1).sendToTarget();
	}

	/**
	 * 返回当前的连接状态
	 */
	public synchronized int getState() {
		return mState;
	}

	/**
	 * 启动服务。特别是在聆听(服务器)模式开始AcceptThread 会话,调用活动onResume()
	 */
	public synchronized void start() {
		if (D)
			Log.d(TAG, "start");

		// 取消任何线程试图连接
		if (mConnectThread != null) {
			mConnectThread.cancel();
			mConnectThread = null;
		}
		// 在运行一个连接时取消任何线程
		if (mConnectedThread != null) {
			mConnectedThread.cancel();
			mConnectedThread = null;
		}

		// 启动线程来监听BluetoothServerSocket
		if (mAcceptThread == null) {
			mAcceptThread = new AcceptThread();
			mAcceptThread.start();
		}
		setState(STATE_LISTEN);
	}

	/**
	 * 启动ConnectThread发起连接到一个远程设备。 device BluetoothDevice的连接
	 */
	public synchronized void connect(BluetoothDevice device) {
		if (D)
			Log.d(TAG, "connect to:" + device);

		// 取消任何线程试图连接 
		if (mState == STATE_CONNECTING) {
			if (mConnectThread != null) {
				mConnectThread.cancel();
				mConnectThread = null;
			}
		}
		// 正在运行一个连接时取消任何线程
		if (mConnectedThread != null) {
			mConnectedThread.cancel();
			mConnectedThread = null;
		}
		// 启动线程与给定的设备
		mConnectThread = new ConnectThread(device);
		mConnectThread.start();
		setState(STATE_CONNECTING);

	}

	/**
	 * 启动ConnectedThread开始管理一个蓝牙连接
	 * 
	 * @param socket
	 *            BluetoothSocket上的连接
	 * @param device
	 *            这个BluetoothDevice已经连接
	 */
	public synchronized void connected(BluetoothSocket socket,
			BluetoothDevice device) {
		if (D)
			Log.d(TAG, "Connected");
		// 取消线程完成连接
		if (mConnectThread != null) {
			mConnectThread.cancel();
			mConnectThread = null;
		}
		// 正在运行一个连接取消任何线程
		if (mConnectedThread != null) {
			mConnectedThread.cancel();
			mConnectedThread = null;
		}
		// 取消接受线程,因为只是想连接到一个设备
		if (mAcceptThread != null) {
			mAcceptThread.cancel();
			mAcceptThread = null;
		}
		// 启动线程来管理连接和执行传输
		mConnectedThread = new ConnectedThread(socket);
		mConnectedThread.start();

		// 发送连接设备的名称回到UI活动
		Message msg = mHandler.obtainMessage(MessageConstant.MESSAGE_DEVICE_NAME);
		Bundle bundle = new Bundle();
		bundle.putString(MessageConstant.DEVICE_NAME, device.getName());
		msg.setData(bundle);
		mHandler.sendMessage(msg);

		setState(STATE_CONNECTED);
	}

	/**
	 * 停止所有的线程
	 */
	public synchronized void stop() {
		if (D)
			Log.d(TAG, "stop");
		setState(STATE_NONE);
		if (mConnectThread != null) {
			mConnectThread.cancel();
			mConnectThread = null;
		}
		if (mConnectedThread != null) {
			mConnectedThread.cancel();
			mConnectedThread = null;

		}
		if (mAcceptThread != null) {
			mAcceptThread.cancel();
			mAcceptThread = null;
		}
	}

	/**
	 * 在一个非同步的ConnectedThread方式下写
	 * 
	 * @param out
	 *            写的字节
	 */
	public void write(byte[] out) {
		// 创建临时对象
	//  public void write(byte[] out) {
		ConnectedThread r;
		// ConnectedThread同步的副本
		synchronized (this) {
			if (mState != STATE_CONNECTED)
				return;
			r = mConnectedThread;
		}
		// 执行写不同步
		r.write(out);
	}

	/**
	 * 启动ConnectThread发起连接到一个远程设备
	 * 
	 * @param device
	 *            BluetoothDevice的连接
	 */

	/**
	 * 连接请求失败并通知UI活动
	 */
	private void connectionFailed() {
		setState(STATE_LISTEN);
		// 一条失败消息发送回活动

		Message msg = mHandler.obtainMessage(MessageConstant.MESSAGE_TOAST);
		Bundle bundle = new Bundle();
		bundle.putString(MessageConstant.TOAST, "不能连接到设备！");
		msg.setData(bundle);
		mHandler.sendMessage(msg);

	}

	/**
	 * 表示这个连接丢失并通知UI活动。
	 */
	private void connectionLost() {
		// 一条失败消息发送给Activity
		Message msg = mHandler.obtainMessage(MessageConstant.MESSAGE_TOAST);
		Bundle bundle = new Bundle();
		bundle.putString(MessageConstant.TOAST, "已断开与打印机的连接！");
		msg.setData(bundle);
		mHandler.sendMessage(msg);
	}

	/**
	 * 这个线程运行监听传入的连接,它像一个服务器端客户机,它运行,直到一个连接被接受(或者,直到被取消)。
	 */
	private class AcceptThread extends Thread {
		// 本地服务器Socket
		private final BluetoothServerSocket mmServerSocket;

		public AcceptThread() {
			BluetoothServerSocket tmp = null;
			// 创建一个新的监听服务器Socket
			try {
				tmp = mAdapter
						.listenUsingRfcommWithServiceRecord(NAME, MY_UUID);
			} catch (IOException e) {
				// TODO 自动生成的catch块
				Log.e(TAG, "listen() failed", e);
			}
			mmServerSocket = tmp;

		}

		public void run() {
			if (D)
				Log.d(TAG, "BEGIN mAcceptThread" + this);
			setName("AcceptThread");
			BluetoothSocket socket = null;

			// 如果是不连接的听服务器Socket

			while (mState != STATE_CONNECTED) {
				try {
					// 一个阻塞调用,只会返回成功连接或一个例外
					socket = mmServerSocket.accept();
				} catch (IOException e) {
					Log.e(TAG, "accept() failed", e);
					break;
				}

				// 如果连接被接受
				if (socket != null) {
					synchronized (BluetoothService.this) {
						switch (mState) {
						case STATE_LISTEN:
						case STATE_CONNECTING:
							// 情况正常,启动连接的线程。
							connected(socket, socket.getRemoteDevice());
							break;
						case STATE_NONE:
						case STATE_CONNECTED:
							// 要么没有准备好或已经连接,终止新的 socket.
							try {
								socket.close();
							} catch (IOException e) {
								Log.e(TAG, "Could not close unwanted socket", e);
							}
							break;   
						}
					}
				}
			}
			if (D)
				Log.i(TAG, "END mAcceptThread");
		}

		public void cancel() {
			if (D)
				Log.d(TAG, "cancel " + this);
			try {
				mmServerSocket.close();
			} catch (IOException e) {
				Log.e(TAG, "close() of server failed", e);
			}

		}

	}

	/**
	 * 启动ConnectedThread开始管理一个蓝牙连接 socket BluetoothSocket上的连接 device
	 * 装置的BluetoothDevice已经连接
	 */

	/**
	 * 这个线程运行试图使一个外向连接一个设备。它运行直通要么连接成功要么失败。
	 */
	private class ConnectThread extends Thread {
		private final BluetoothSocket mmSocket;
		private final BluetoothDevice mmDevice;

		public ConnectThread(BluetoothDevice device) {
			mmDevice = device;
			BluetoothSocket tmp = null;
			// 为一个连接的BluetoothDevice得到一个BluetoothSocket
			try {
				tmp = device.createRfcommSocketToServiceRecord(MY_UUID);
			} catch (IOException e) {
				Log.e(TAG, "create failed", e);
			}
			mmSocket = tmp;
		}

		public void run() {
			Log.i(TAG, "BEFIN mConnectThrread");
			setName("ConnectThread");

			mAdapter.cancelDiscovery();

			// 让BluetoothSocket的连接

			try {
				// 一个阻塞调用返回成功连接或一个例外
				mmSocket.connect();
			} catch (IOException e) {

				connectionFailed();
				// 关闭socket
				try {
					mmSocket.close();
				} catch (IOException e2) {
					Log.e(TAG,
							"unable to close() socket during connection failure",
							e2);
				}
				// 启动服务到重启的听力模式
				BluetoothService.this.start();
				return;
			}
			// 重置ConnectThread
			synchronized (BluetoothService.this) {
				mConnectThread = null;

			}
			// 启动连接的线程
			connected(mmSocket, mmDevice);

		}

		public void cancel() {
			try {
				mmSocket.close();
			} catch (IOException e) {
				Log.e(TAG, "close() of connect socket failed", e);
			}
		}
	}

	/**
	 * 这个线程运行在连接一个远程的设备上 它处理所有传入和传出的传输。
	 */
	private class ConnectedThread extends Thread {
		private final BluetoothSocket mmSocket;
		private final InputStream mmInStream;
		private final OutputStream mmOutStream;

		public ConnectedThread(BluetoothSocket socket) {
			Log.d(TAG, "create ConnectedThread");
			mmSocket = socket;
			InputStream tmpIn = null;
			OutputStream tmpOut = null;

			// BluetoothSocket输入和输出流
			try {
				tmpIn = socket.getInputStream();
				tmpOut = socket.getOutputStream();
			} catch (IOException e) {
				Log.e(TAG, "temp sockets not created", e);
			}

			mmInStream = tmpIn;
			mmOutStream = tmpOut;
		}

		public void run() {
			Log.i(TAG, "BEGIN mConnectedThread");
			int bytes;
			while (true) {
				
				try {
					byte[] buffer = new byte[256];
					// 从InputStream中阅读  
					bytes = mmInStream.read(buffer);
					if (bytes > 0) {
						// 发送获得的字节到UI的 Activity
						
						for (int i=0;i<bytes;i++)
			    		{
			    			BluetoothRecvBuf[BluetoothRecvBufLen++] = buffer[i];
			    			
			    			Log.i(TAG,"read:"  +buffer[i]+"::");
			    		}
						returnStr=new String(BluetoothRecvBuf, 0, BluetoothRecvBufLen);
						Log.i(TAG,"返回："+returnStr);
						//mHandler.obtainMessage(MessageConstant.MESSAGE_READ,BluetoothRecvBufLen, -1, BluetoothRecvBuf).sendToTarget();
					}
					
				
					 
				} catch (IOException e) {
					Log.e(TAG, "disconnected", e);
					connectionLost();

					if (mState != STATE_NONE) {

						BluetoothService.this.start();
					}
					break;
				}
				
			}
		}

		/**
		 * OutStream写入连接
		 * 
		 * @param buffer
		 *            写的字节
		 */
		public void write(byte[] buffer) {
			try {
				mmOutStream.write(buffer,0,buffer.length);

				// 分享所发送的消息回到UI的Activity
				mHandler.obtainMessage(MessageConstant.MESSAGE_WRITE, -1, -1,
						buffer).sendToTarget();
			} catch (IOException e) {
				Log.e(TAG, "Exception during write", e);
			}
		}
  
		public void cancel() {
			try {
				mmSocket.close();
			} catch (IOException e) {
				Log.e(TAG, "close() of connect socket failed", e);
			}
		}
		
	}

	public void pushCmd(byte n){
		byte[] bytes = new byte[1];
		bytes[0] = n;
		write(bytes);
	}
	public void pushCmd(byte[] bytes) {
		
		BluetoothRecvBufLen = 0;
	    returnStr = "";
		write(bytes);
	
}
    
}
